#include <Wire.h>
#include <Pixy2.h>
#include <AccelStepper.h>
#include <MultiStepper.h>

Pixy2 pixy;

AccelStepper stepperX(1, 2, 5);
AccelStepper stepperY(1, 3, 6);

MultiStepper steppersControl;

long gotoposition[2];

//PID gains (THESE VALUES ARE ARBITRARY)
float xkp = -0.7; 
float xki = -0.0001;
float xkd = -100;

float ykp = -0.7;
float yki = -0.0001;
float ykd = -100;


int cycleTime = 50; //this can also be changed if you want the arduino to process more values

int xtarget = 150; //Changing these will change the position at which the ball will come to rest.
int ytarget = 100;


int xerror = 0;
float xlastError = 0;
int yerror = 0;
float ylastError = 0;

float xp = 0;
float xi = 0;
float xd = 0;
float yp = 0;
float yi = 0;
float yd = 0;

float xangle = 0;
float yangle = 0;


void setup() {

  Serial.begin(115200);
 
 pixy.init();
 pixy.setLamp(1,1); //If you don't want the lights on the pixy on just delete this

  delay(1000);
  
  stepperX.setMaxSpeed(1000); // Set maximum speed value for the stepper
  stepperY.setMaxSpeed(1000);

  steppersControl.addStepper(stepperX);
  steppersControl.addStepper(stepperY);

  //homing sequence
  gotoposition[0] = 120;
  gotoposition[1] = -90;
  steppersControl.moveTo(gotoposition); // Calculates the required speed for all motors
  steppersControl.runSpeedToPosition(); // Blocks until all steppers are in position
  stepperX.setCurrentPosition(0);
  stepperY.setCurrentPosition(0);
}

void loop() {
  int i; 
 
  pixy.ccc.getBlocks();
  
  
  if (pixy.ccc.numBlocks)
  {  
    //Proportional  
    xerror = xtarget - pixy.ccc.blocks[i].m_x;
    xp = xerror * xkp;
    yerror = ytarget - pixy.ccc.blocks[i].m_y;
    yp = yerror * ykp;

    //Integral
    xi = (xerror*xki*cycleTime) + xi;
    yi = (yerror*yki*cycleTime) + yi;
    
    //Derivative
    xd = xkd*(xerror - xlastError)/cycleTime;
    yd = ykd*(yerror - ylastError)/cycleTime;

    //Output
    xangle = xp+xi+xd;
    yangle = yp+yi+yd;

    xlastError = xerror;
    ylastError = yerror;

    gotoposition[0] = xangle;
    gotoposition[1] = yangle;
    steppersControl.moveTo(gotoposition); // Calculates the required speed for all motors
    steppersControl.runSpeedToPosition(); // Blocks until all steppers are in position
    
    //printing the output values, useful for debugging
    Serial.print(xangle);
    Serial.print(", ");
    Serial.println(yangle);
    delay(cycleTime);
    }
}
